import logging
import sys
import unittest

from geoalchemy2.shape import from_shape
from shapely.geometry.point import Point

from peek.core.orm.Display import DispLevel
from peek.core.queue_processesors.GridKeyUtil import makeGridKeys

logging.basicConfig(
    format="%(asctime)s %(levelname)s %(name)s:%(message)s",
    datefmt="%d-%b-%Y %H:%M:%S",
    level=logging.DEBUG,
    stream=sys.stdout,
)

logger = logging.getLogger(__name__)


class MyTestCase(unittest.TestCase):
    def setUp(self):
        self.geom = from_shape(Point((0, 0)))

    def test_keys(self):
        def make(min, max):
            gridKeys = makeGridKeys(self.geom, DispLevel(minZoom=min, maxZoom=max))
            return sorted(gridKeys)

        # Test Level 0 only
        self.assertEqual(make(min=0.0, max=0.0), [])

        # Test Level 0 only
        self.assertEqual(make(min=0.0, max=0.04), ["0.0x0"])

        # Test Level 0 only
        self.assertEqual(make(min=0.0, max=0.1), ["0.0x0", "1.0x0"])

        # TEST Z 0, 1
        self.assertEqual(make(min=0.0, max=0.3), ["0.0x0", "1.0x0", "2.0x0"])

        # TEST Z 0, 1
        self.assertEqual(make(min=0.1, max=0.4), ["1.0x0", "2.0x0"])

        # TEST Z 0, 1
        self.assertEqual(make(min=0.1, max=0.5), ["1.0x0", "2.0x0"])

        # TEST Z 1 only
        self.assertEqual(make(min=0.1001, max=0.5), ["2.0x0"])

        # TEST Z 1 only
        self.assertEqual(make(min=0.23, max=0.499), ["2.0x0"])

        # TEST Z 0, 1, 2
        self.assertEqual(make(min=0.1, max=0.5001), ["1.0x0", "2.0x0", "3.0x0"])

        # TEST Z 0, 1, 2
        self.assertEqual(make(min=0.0, max=9999), ["0.0x0", "1.0x0", "2.0x0", "3.0x0"])

        # TEST Z 0, 1, 2
        self.assertEqual(make(min=0.0, max=0.7), ["0.0x0", "1.0x0", "2.0x0", "3.0x0"])

        # TEST Z 1, 2
        self.assertEqual(make(min=0.25, max=0.7), ["2.0x0", "3.0x0"])

        # TEST Z 1, 2
        self.assertEqual(make(min=0.25, max=0.7), ["2.0x0", "3.0x0"])

        # TEST Z 2
        self.assertEqual(make(min=0.5, max=0.7), ["2.0x0", "3.0x0"])

        # TEST Z 2
        self.assertEqual(make(min=0.5001, max=0.7), ["3.0x0"])

        # TEST Z 2
        self.assertEqual(make(min=0.5001, max=0.7), ["3.0x0"])



"""add disp action

Peek Plugin Database Migration Script

Revision ID: 42a8dc25e588
Revises: f697025b2013
Create Date: 2020-05-10 15:20:02.102210

"""

# revision identifiers, used by Alembic.
revision = "42a8dc25e588"
down_revision = "f697025b2013"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispBase",
        sa.Column("action", sa.SmallInteger(), nullable=True),
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DispBase", "action", schema="pl_diagram")
    # ### end Alembic commands ###



import logging

from peek_plugin_diagram._private.server.api.DiagramImportApi import DiagramImportApi
from peek_plugin_diagram._private.server.api.DiagramViewerApi import DiagramViewerApi
from peek_plugin_diagram._private.server.controller.BranchUpdateController import (
    BranchUpdateController,
)
from peek_plugin_diagram._private.server.controller.DispImportController import (
    DispImportController,
)
from peek_plugin_diagram._private.server.controller.LookupImportController import (
    LookupImportController,
)
from peek_plugin_diagram._private.server.controller.StatusController import (
    StatusController,
)
from peek_plugin_diagram.server.DiagramApiABC import DiagramApiABC
from peek_plugin_diagram.server.DiagramImportApiABC import DiagramImportApiABC
from peek_plugin_diagram.server.DiagramViewerApiABC import DiagramViewerApiABC

logger = logging.getLogger(__name__)


class DiagramApi(DiagramApiABC):
    def __init__(
        self,
        mainController: StatusController,
        dispImportController: DispImportController,
        lookupImportController: LookupImportController,
        branchUpdateController: BranchUpdateController,
        ormSessionCreator,
    ):
        self._viewerApi = DiagramViewerApi(ormSessionCreator)
        self._importApi = DiagramImportApi(
            mainController,
            dispImportController,
            lookupImportController,
            branchUpdateController,
        )

    def shutdown(self):
        self._importApi.shutdown()
        self._viewerApi.shutdown()

        self._importApi = None
        self._viewerApi = None

    def importApi(self) -> DiagramImportApiABC:
        return self._importApi

    def viewerApi(self) -> DiagramViewerApiABC:
        return self._viewerApi



"""Added CoordSet editable fields

Peek Plugin Database Migration Script

Revision ID: 81c1d2809046
Revises: bf8f559c4df9
Create Date: 2019-05-13 21:08:31.486845

"""

# revision identifiers, used by Alembic.
revision = "81c1d2809046"
down_revision = "bf8f559c4df9"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.alter_column(
        "ModelCoordSet",
        "enabled",
        type_=sa.Boolean(),
        server_default="true",
        schema="pl_diagram",
    )

    op.execute('UPDATE "pl_diagram"."ModelCoordSet" SET "enabled" = true ')

    op.add_column(
        "ModelCoordSet",
        sa.Column("editEnabled", sa.Boolean(), server_default="false", nullable=True),
        schema="pl_diagram",
    )

    op.execute('UPDATE "pl_diagram"."ModelCoordSet" SET "editEnabled" = false ')

    op.alter_column(
        "ModelCoordSet",
        "editEnabled",
        type_=sa.Boolean(),
        nullable=False,
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column(
            "branchesEnabled", sa.Boolean(), server_default="false", nullable=True
        ),
        schema="pl_diagram",
    )

    op.execute('UPDATE "pl_diagram"."ModelCoordSet" SET "branchesEnabled" = false')

    op.alter_column(
        "ModelCoordSet",
        "branchesEnabled",
        type_=sa.Boolean(),
        nullable=False,
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultColorId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultLayerId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultLevelId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultLineStyleId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )

    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultTextStyleId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )

    op.create_index(
        "idxCoordModel_editDefaultColorId",
        "ModelCoordSet",
        ["editDefaultColorId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idxCoordModel_editDefaultLayerId",
        "ModelCoordSet",
        ["editDefaultLayerId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idxCoordModel_editDefaultLevelId",
        "ModelCoordSet",
        ["editDefaultLevelId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idxCoordModel_editDefaultLineStyleId",
        "ModelCoordSet",
        ["editDefaultLineStyleId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idxCoordModel_editDefaultTextStyleId",
        "ModelCoordSet",
        ["editDefaultTextStyleId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_foreign_key(
        "ModelCoordSet_editDefaultLineStyleId_fkey",
        "ModelCoordSet",
        "DispLineStyle",
        ["editDefaultLineStyleId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )

    op.create_foreign_key(
        "ModelCoordSet_editDefaultColorId_fkey",
        "ModelCoordSet",
        "DispColor",
        ["editDefaultColorId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )

    op.create_foreign_key(
        "ModelCoordSet_editDefaultLayerId_fkey",
        "ModelCoordSet",
        "DispLayer",
        ["editDefaultLayerId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )

    op.create_foreign_key(
        "ModelCoordSet_editDefaultLevelId_fkey",
        "ModelCoordSet",
        "DispLevel",
        ["editDefaultLevelId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )

    op.create_foreign_key(
        "ModelCoordSet_editDefaultTextStyleId_fkey",
        "ModelCoordSet",
        "DispTextStyle",
        ["editDefaultTextStyleId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "ModelCoordSet_editDefaultLineStyleId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_constraint(
        "ModelCoordSet_editDefaultColorId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_constraint(
        "ModelCoordSet_editDefaultLayerId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_constraint(
        "ModelCoordSet_editDefaultLevelId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_constraint(
        "ModelCoordSet_editDefaultTextStyleId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )

    op.drop_index(
        "idxCoordModel_editDefaultTextStyleId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_index(
        "idxCoordModel_editDefaultLineStyleId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_index(
        "idxCoordModel_editDefaultLevelId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_index(
        "idxCoordModel_editDefaultLayerId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_index(
        "idxCoordModel_editDefaultColorId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_column("ModelCoordSet", "branchesEnabled", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editEnabled", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultTextStyleId", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultLineStyleId", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultLevelId", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultLayerId", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultColorId", schema="pl_diagram")
    # ### end Alembic commands ###



import logging
from typing import List, Optional

from vortex.DeferUtil import vortexLogFailure
from vortex.rpc.RPC import vortexRPC

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkLoadRpcABC import (
    ACIChunkLoadRpcABC,
)
from peek_plugin_base.PeekVortexUtil import peekServerName, peekBackendNames
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.storage.GridKeyIndex import GridKeyIndexCompiled

logger = logging.getLogger(__name__)


class ClientGridLoaderRpc(ACIChunkLoadRpcABC):
    def __init__(self, liveDbWatchController, dbSessionCreator):
        ACIChunkLoadRpcABC.__init__(self, dbSessionCreator)
        self._liveDbWatchController = liveDbWatchController

    def makeHandlers(self):
        """Make Handlers

        In this method we start all the RPC handlers
        start() returns an instance of it's self so we can simply yield the result
        of the start method.

        """

        yield self.loadGrids.start(funcSelf=self)
        yield self.updateClientWatchedGrids.start(funcSelf=self)
        logger.debug("RPCs started")

    # -------------
    @vortexRPC(
        peekServerName,
        acceptOnlyFromVortex=peekBackendNames,
        timeoutSeconds=120,
        additionalFilt=diagramFilt,
        deferToThread=True,
    )
    def loadGrids(self, offset: int, count: int) -> Optional[bytes]:
        return self.ckiInitialLoadChunksPayloadBlocking(
            offset, count, GridKeyIndexCompiled
        )

    # -------------
    @vortexRPC(
        peekServerName,
        acceptOnlyFromVortex=peekBackendNames,
        additionalFilt=diagramFilt,
    )
    def updateClientWatchedGrids(self, clientId: str, gridKeys: List[str]) -> None:
        """Update Client Watched Grids

        Tell the server that these grids are currently being watched by users.

        :param clientId: A unique identifier of the client (Maybe it's vortex uuid)
        :param gridKeys: A list of grid keys that this client is observing.
        :returns: Nothing
        """

        d = self._liveDbWatchController.updateClientWatchedGrids(clientId, gridKeys)
        d.addErrback(vortexLogFailure, logger, consumeError=True)



from vortex.handler.TupleActionProcessor import TupleActionProcessor

from peek_plugin_diagram._private.PluginNames import diagramActionProcessorName
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.server.controller.BranchLiveEditController import (
    BranchLiveEditController,
)
from peek_plugin_diagram._private.server.controller.BranchUpdateController import (
    BranchUpdateController,
)
from peek_plugin_diagram._private.tuples.branch.BranchLiveEditTupleAction import (
    BranchLiveEditTupleAction,
)
from peek_plugin_diagram._private.tuples.branch.BranchUpdateTupleAction import (
    BranchUpdateTupleAction,
)
from .controller.StatusController import StatusController


def makeTupleActionProcessorHandler(
    statusController: StatusController,
    branchUpdateController: BranchUpdateController,
    branchLiveEditController: BranchLiveEditController,
):
    processor = TupleActionProcessor(
        tupleActionProcessorName=diagramActionProcessorName,
        additionalFilt=diagramFilt,
        defaultDelegate=statusController,
    )

    processor.setDelegate(BranchUpdateTupleAction.tupleName(), branchUpdateController)

    processor.setDelegate(
        BranchLiveEditTupleAction.tupleName(), branchLiveEditController
    )

    return processor



"""Added Poly template name

Peek Plugin Database Migration Script

Revision ID: a86e0317e22a
Revises: ec97d31aebb0
Create Date: 2019-09-29 22:24:23.878249

"""

# revision identifiers, used by Alembic.
revision = "a86e0317e22a"
down_revision = "ec97d31aebb0"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispPolyline",
        sa.Column("targetEdgeTemplateId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "DispPolyline",
        sa.Column("targetEdgeTemplateName", sa.String(), nullable=True),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolyline_targetEdgeTemplateId",
        "DispPolyline",
        ["targetEdgeTemplateId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_foreign_key(
        "DispPolyline_targetEdgeTemplateId_fkey",
        "DispPolyline",
        "DispEdgeTemplate",
        ["targetEdgeTemplateId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
        ondelete="SET NULL",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "DispPolyline_targetEdgeTemplateId_fkey",
        "DispPolyline",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_index(
        "idx_DispPolyline_targetEdgeTemplateId",
        table_name="DispPolyline",
        schema="pl_diagram",
    )
    op.drop_column("DispPolyline", "targetEdgeTemplateName", schema="pl_diagram")
    op.drop_column("DispPolyline", "targetEdgeTemplateId", schema="pl_diagram")
    # ### end Alembic commands ###



import logging
import time

from peek_plugin_diagram._private.worker.tasks.ImportDispTask import importDispsTask
from peek_plugin_livedb.server.LiveDBWriteApiABC import LiveDBWriteApiABC
from twisted.internet import reactor
from twisted.internet.defer import inlineCallbacks, Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger

logger = logging.getLogger(__name__)


class DispImportController:
    def __init__(self, liveDbWriteApi: LiveDBWriteApiABC):
        self._liveDbWriteApi = liveDbWriteApi

    def shutdown(self):
        self._liveDbWriteApi = None

    @inlineCallbacks
    def importDisps(
        self,
        modelSetKey: str,
        coordSetKey: str,
        importGroupHash: str,
        dispsEncodedPayload: bytes,
    ):
        liveDbItemsToImport = yield importDispsTask.delay(
            modelSetKey, coordSetKey, importGroupHash, dispsEncodedPayload
        )

        if liveDbItemsToImport:
            yield self._liveDbWriteApi.importLiveDbItems(
                modelSetKey, liveDbItemsToImport
            )

            # Give and connector plugins time to load the new items
            yield self._sleep(2.0)

            yield self._liveDbWriteApi.pollLiveDbValueAcquisition(
                modelSetKey, [i.key for i in liveDbItemsToImport]
            )

    def _sleep(self, seconds):
        d = Deferred()
        reactor.callLater(seconds, d.callback, True)
        return d



"""Added cascade constraints

Peek Plugin Database Migration Script

Revision ID: 5e2f84d757f2
Revises: cea17a52b9ae
Create Date: 2018-05-03 16:49:27.699014

"""

# revision identifiers, used by Alembic.
revision = "5e2f84d757f2"
down_revision = "cea17a52b9ae"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "GridKeyIndex_coordSetId_fkey",
        "GridKeyIndex",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.create_foreign_key(
        "GridKeyIndex_coordSetId_fkey",
        "GridKeyIndex",
        "ModelCoordSet",
        ["coordSetId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
        ondelete="CASCADE",
    )
    op.drop_constraint(
        "LiveDbDispLink_coordSetId_fkey",
        "LiveDbDispLink",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.create_foreign_key(
        "LiveDbDispLink_coordSetId_fkey",
        "LiveDbDispLink",
        "ModelCoordSet",
        ["coordSetId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
        ondelete="CASCADE",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "LiveDbDispLink_coordSetId_fkey",
        "LiveDbDispLink",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.create_foreign_key(
        "LiveDbDispLink_coordSetId_fkey",
        "LiveDbDispLink",
        "ModelCoordSet",
        ["coordSetId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )
    op.drop_constraint(
        "GridKeyIndex_coordSetId_fkey",
        "GridKeyIndex",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.create_foreign_key(
        "GridKeyIndex_coordSetId_fkey",
        "GridKeyIndex",
        "ModelCoordSet",
        ["coordSetId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )
    # ### end Alembic commands ###



import logging
from typing import Union

from twisted.internet.defer import Deferred, inlineCallbacks
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.server.controller.BranchLiveEditController import (
    BranchLiveEditController,
)

logger = logging.getLogger(__name__)


class BranchLiveEditTupleProvider(TuplesProviderABC):
    def __init__(self, branchLiveEditController: BranchLiveEditController):
        self._controller = branchLiveEditController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        coordSetId = tupleSelector.selector.get("coordSetId")
        key = tupleSelector.selector.get("key")

        tuple = self._controller.getLiveEditTuple(coordSetId, key)
        tuples = [tuple] if tuple else []

        payloadEnvelope = yield Payload(filt, tuples=tuples).makePayloadEnvelopeDefer()
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg



from typing import Optional

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ImportDispTextStyleTuple(Tuple):
    """Import Display Text Style Tuple"""

    __tupleType__ = diagramTuplePrefix + "ImportDispTextStyleTuple"

    name: str = TupleField()

    fontName: str = TupleField()
    fontSize: float = TupleField()

    STYLE_BOLD = "bold"
    fontStyle: str = TupleField()

    scalable: bool = TupleField()
    scaleFactor: float = TupleField()

    importHash: str = TupleField()

    modelSetKey: str = TupleField()



from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ImportDispColorTuple(Tuple):
    """Imported Display Color

    This tuple is used by other plugins to load colours into the diagram.

    """

    __tupleType__ = diagramTuplePrefix + "ImportDispColorTuple"

    #:  The name of the color
    name: str = TupleField()

    #:  The color
    color: str = TupleField()

    #:  The alt color
    altColor: str = TupleField()

    #:  The swap period if this is a flashing colour
    swapPeriod: float = TupleField()

    #:  The name of the model set for this colour
    modelSetKey: str = TupleField()

    #:  The import hash for this colour
    importHash: str = TupleField()



import logging

from peek_plugin_diagram._private.storage.Display import DispText
from peek_plugin_diagram._private.storage.LiveDbDispLink import (
    LIVE_DB_KEY_DATA_TYPE_BY_DISP_ATTR,
)
from peek_plugin_livedb.tuples.ImportLiveDbItemTuple import ImportLiveDbItemTuple

logger = logging.getLogger(__name__)


def _mergeInLiveDbValues(disp, liveDbItemByKey):
    for dispLink in disp.liveDbLinks:
        liveDbItem = liveDbItemByKey.get(dispLink.liveDbKey)
        if liveDbItem:
            _mergeInLiveDbValue(disp, dispLink, liveDbItem)


def _mergeInLiveDbValue(disp, dispLink, liveDbItem, value=None):
    # This allows us to change the value and use recursion a little
    # (Value is converted to different data types and recursively called, see below)
    if value is None:
        value = liveDbItem.displayValue

    # At least for colors and string :
    # If the color vale is None, set the attribute to None as well
    # At this stage we don't expect other data types to be None
    if value is None:
        keyType = LIVE_DB_KEY_DATA_TYPE_BY_DISP_ATTR[dispLink.dispAttrName]

        if keyType == ImportLiveDbItemTuple.DATA_TYPE_LINE_WIDTH:
            setattr(disp, dispLink.dispAttrName, 0)
            return

        nullableTypes = (
            ImportLiveDbItemTuple.DATA_TYPE_COLOR,
            ImportLiveDbItemTuple.DATA_TYPE_STRING_VALUE,
        )

        assert keyType in nullableTypes, (
            "keyType |%s| is not DATA_TYPE_COLOR\n"
            "==DISP LINK== %s\n"
            "==LIVE DB ITEM== %s" % (keyType, dispLink, liveDbItem)
        )

        setattr(disp, dispLink.dispAttrName, None)
        return

    # ----------------------------
    # Not text
    if not (isinstance(disp, DispText) and dispLink.dispAttrName == "text"):
        setattr(disp, dispLink.dispAttrName, value)
        return

    # ----------------------------
    # Special case for Text

    # If there is no format, then just use the value
    if not disp.textFormat:
        disp.text = value
        return

    # The value in the livedb has not yet been acquired from the external system
    # Just return the formatting, at least the text will show something.
    if "%" in disp.textFormat and disp.textFormat == value:
        disp.text = disp.textFormat
        return

    _applyTextFormat(disp, dispLink, liveDbItem, value)


def _applyTextFormat(disp, dispLink, liveDbItem, value):
    # If there is a format, then convert it.

    try:
        disp.text = disp.textFormat % value

    except TypeError as e:
        message = str(e)
        # Lazy format type detection
        try:
            if "number is required" in message:
                return _applyTextFormat(disp, dispLink, liveDbItem, float(value))

            if "invalid literal for int" in message:
                return _applyTextFormat(disp, dispLink, liveDbItem, int(value))

            if "an integer is required, not str" in message:
                return _applyTextFormat(disp, dispLink, liveDbItem, int(value))

            if "float is required" in message:
                return _applyTextFormat(disp, dispLink, liveDbItem, float(value))

            if "must be real number, not str" in message:
                return _applyTextFormat(disp, dispLink, liveDbItem, float(value))

            if "could not convert string to float" in message:
                return _applyTextFormat(disp, dispLink, liveDbItem, float(value))

        except ValueError as e:
            disp.text = disp.textFormat

        logger.warning(
            "DispText %s, %s textFormat=|%s| value=|%s| failed with %s\n"
            "This can occur if the LiveDbItem.rawValue has not yet been updated",
            disp.id,
            liveDbItem.key,
            disp.textFormat,
            value,
            message,
        )



import logging
from collections import defaultdict
from datetime import datetime
from typing import List, Set, Tuple, Dict

import pytz
from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_diagram._private.server.controller.DispCompilerQueueController import (
    DispCompilerQueueController,
)
from peek_plugin_diagram._private.storage.Display import DispBase
from peek_plugin_diagram._private.storage.GridKeyIndex import (
    GridKeyIndex,
    GridKeyCompilerQueue,
)
from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet, ModelSet
from peek_plugin_diagram._private.storage.branch.BranchIndex import BranchIndex
from peek_plugin_diagram._private.storage.branch.BranchIndexCompilerQueue import (
    BranchIndexCompilerQueue,
)
from peek_plugin_diagram._private.tuples.branch.BranchTuple import BranchTuple
from peek_plugin_base.worker.CeleryApp import celeryApp
from peek_plugin_diagram._private.worker.tasks.ImportDispTask import _bulkInsertDisps
from peek_plugin_diagram._private.worker.tasks.branch.BranchDispUpdater import (
    _deleteBranchDisps,
    _convertBranchDisps,
)
from peek_plugin_diagram._private.worker.tasks.branch._BranchIndexCalcChunkKey import (
    makeChunkKeyForBranchIndex,
)
from sqlalchemy import select, and_, bindparam
from sqlalchemy.orm.exc import NoResultFound
from txcelery.defer import DeferrableTask
from vortex.Payload import Payload

logger = logging.getLogger(__name__)


@DeferrableTask
@celeryApp.task(bind=True)
def updateBranches(self, modelSetId: int, branchEncodedPayload: bytes) -> None:
    """Update Branch

    This method is called from the UI to update a single branch.
    It could be called from a server API as well.

    All the branches must be for the same model set.

    """
    # Decode BranchTuples payload
    updatedBranches: List[BranchTuple] = (
        Payload().fromEncodedPayload(branchEncodedPayload).tuples
    )

    startTime = datetime.now(pytz.utc)

    queueTable = BranchIndexCompilerQueue.__table__
    dispBaseTable = DispBase.__table__
    gridKeyIndexTable = GridKeyIndex.__table__

    gridKeyCompilerQueueTable = GridKeyCompilerQueue.__table__

    branchesByCoordSetId: Dict[int, List[BranchTuple]] = defaultdict(list)
    chunkKeys: Set[str] = set()

    newBranchesToInsert = []

    # Create a lookup of CoordSets by ID
    dbSession = CeleryDbConn.getDbSession()
    try:
        # Get the latest lookups
        modelSet = dbSession.query(ModelSet).filter(ModelSet.id == modelSetId).one()
        coordSetById = {i.id: i for i in dbSession.query(ModelCoordSet).all()}
        dbSession.expunge_all()

        # Update the branches
        # This will be a performance problem if lots of branches are updated,
        # however, on first writing this will just be used by the UI for updating
        # individual branches.
        for branch in updatedBranches:
            try:
                if str(branch.id).startswith("NEW_"):
                    branch.id = None

                if branch.id is None:
                    branchIndex = (
                        dbSession.query(BranchIndex)
                        .filter(BranchIndex.coordSetId == branch.coordSetId)
                        .filter(BranchIndex.key == branch.key)
                        .one()
                    )
                else:
                    branchIndex = (
                        dbSession.query(BranchIndex)
                        .filter(BranchIndex.id == branch.id)
                        .one()
                    )
                branch.id = branchIndex.id
                branchIndex.packedJson = branch.packJson()
                branchIndex.updatedDate = branch.updatedDate

            except NoResultFound:
                newBranchesToInsert.append(branch)

            branchesByCoordSetId[branch.coordSetId].append(branch)

            chunkKeys.add(makeChunkKeyForBranchIndex(modelSet.key, branch.key))

        dbSession.commit()

    except Exception as e:
        dbSession.rollback()
        logger.debug("Retrying updateBranch, %s", e)
        logger.exception(e)
        raise self.retry(exc=e, countdown=3)

    finally:
        dbSession.close()

    dbSession = CeleryDbConn.getDbSession()

    try:
        if newBranchesToInsert:
            _insertOrUpdateBranches(
                dbSession, modelSet.key, modelSet.id, newBranchesToInsert
            )
            dbSession.commit()

        # Make an array of all branch IDs
        allBranchIds = []
        for branches in branchesByCoordSetId.values():
            allBranchIds.extend([b.id for b in branches])

        # Find out all the existing grids effected by this branch.
        gridsToRecompile = dbSession.execute(
            select(
                distinct=True,
                columns=[gridKeyIndexTable.c.gridKey, gridKeyIndexTable.c.coordSetId],
                whereclause=dispBaseTable.c.branchId.in_(allBranchIds),
            ).select_from(gridKeyIndexTable.join(dispBaseTable))
        ).fetchall()

        allNewDisps = []
        allDispIdsToCompile = []

        packedJsonUpdates = []
        # Recompile the BranchGridIndexes
        for coordSetId, branches in branchesByCoordSetId.items():
            coordSet = coordSetById[coordSetId]
            assert coordSet.modelSetId == modelSetId, "Branches not all from one model"

            newDisps, dispIdsToCompile = _convertBranchDisps(branches)
            allNewDisps.extend(newDisps)
            allDispIdsToCompile.extend(dispIdsToCompile)

            packedJsonUpdates.extend(
                [dict(b_id=b.id, b_packedJson=b.packJson()) for b in branches]
            )

        dbSession.execute(
            dispBaseTable.delete(dispBaseTable.c.branchId.in_(allBranchIds))
        )

        dbSession.commit()

        # NO TRANSACTION
        # Bulk load the Disps
        _bulkInsertDisps(CeleryDbConn.getDbEngine(), allNewDisps)

        # Queue the compiler
        DispCompilerQueueController.queueDispIdsToCompileWithSession(
            allDispIdsToCompile, dbSession
        )

        # Update the JSON again back into the grid index.
        stmt = (
            BranchIndex.__table__.update()
            .where(BranchIndex.__table__.c.id == bindparam("b_id"))
            .values(packedJson=bindparam("b_packedJson"))
        )
        dbSession.execute(stmt, packedJsonUpdates)

        # 3) Queue chunks for recompile
        dbSession.execute(
            queueTable.insert(),
            [dict(modelSetId=modelSetId, chunkKey=c) for c in chunkKeys],
        )

        # 4) Queue chunks for
        if gridsToRecompile:
            dbSession.execute(
                gridKeyCompilerQueueTable.insert(),
                [
                    dict(coordSetId=item.coordSetId, gridKey=item.gridKey)
                    for item in gridsToRecompile
                ],
            )

        dbSession.commit()

        logger.debug(
            "Updated %s BranchIndexes queued %s chunks in %s",
            len(updatedBranches),
            len(chunkKeys),
            (datetime.now(pytz.utc) - startTime),
        )

    except Exception as e:
        dbSession.rollback()
        logger.debug("Retrying updateBranch, %s", e)
        logger.exception(e)
        raise self.retry(exc=e, countdown=3)

    finally:
        dbSession.close()


@DeferrableTask
@celeryApp.task(bind=True)
def removeBranches(self, modelSetKey: str, coordSetKey: str, keys: List[str]) -> None:
    """Remove Branches

    This worker task removes branches from the indexes.

    """

    startTime = datetime.now(pytz.utc)

    branchIndexTable = BranchIndex.__table__
    queueTable = BranchIndexCompilerQueue.__table__

    # Create a lookup of CoordSets by ID
    dbSession = CeleryDbConn.getDbSession()
    try:
        coordSet = (
            dbSession.query(ModelCoordSet)
            .filter(ModelCoordSet.modelSet.key == modelSetKey)
            .filter(ModelCoordSet.key == coordSetKey)
            .one()
        )

        dbSession.expunge_all()

    finally:
        dbSession.close()

    engine = CeleryDbConn.getDbEngine()
    conn = engine.connect()
    transaction = conn.begin()

    try:
        items = conn.execute(
            select(
                distinct=True,
                columns=[branchIndexTable.c.id, branchIndexTable.c.chunkKey],
                whereclause=and_(
                    branchIndexTable.c.key.in_(keys),
                    branchIndexTable.c.coordSetId == coordSet.id,
                ),
            )
        ).fetchall()

        branchIndexIds = [i.id for i in items]
        chunkKeys = set([i.chunkKey for i in items])

        _deleteBranchDisps(conn, branchIndexIds)

        # 1) Delete existing branches
        conn.execute(branchIndexTable.delete(branchIndexTable.c.id.in_(branchIndexIds)))

        # 3) Queue chunks for recompile
        conn.execute(
            queueTable.insert(),
            [dict(modelSetId=coordSet.modelSetId, chunkKey=c) for c in chunkKeys],
        )

        transaction.commit()
        logger.debug(
            "Deleted %s BranchIndexes queued %s chunks in %s",
            len(branchIndexIds),
            len(chunkKeys),
            (datetime.now(pytz.utc) - startTime),
        )

    except Exception as e:
        transaction.rollback()
        logger.debug("Retrying createOrUpdateBranches, %s", e)
        logger.exception(e)
        raise self.retry(exc=e, countdown=3)

    finally:
        conn.close()


def _insertOrUpdateBranches(
    conn, modelSetKey: str, modelSetId: int, newBranches: List[BranchTuple]
) -> None:
    """Insert or Update Branches

    1) Delete existing branches
    2) Insert new branches
    3) Queue chunks for recompile

    """

    startTime = datetime.now(pytz.utc)

    branchIndexTable = BranchIndex.__table__
    queueTable = BranchIndexCompilerQueue.__table__

    importHashSet = set()

    chunkKeysForQueue: Set[Tuple[int, str]] = set()

    # Get the IDs that we need
    newIdGen = CeleryDbConn.prefetchDeclarativeIds(BranchIndex, len(newBranches))

    # Create state arrays
    inserts = []

    # Work out which objects have been updated or need inserting
    for newBranch in newBranches:
        importHashSet.add(newBranch.importGroupHash)

        # noinspection PyTypeChecker
        newBranch.id = next(newIdGen)
        branchJson = newBranch.packJson()

        existingObject = BranchIndex(
            id=newBranch.id,
            coordSetId=newBranch.coordSetId,
            key=newBranch.key,
            updatedDate=newBranch.updatedDate,
            createdDate=newBranch.createdDate,
            importHash=newBranch.importHash,
            importGroupHash=newBranch.importGroupHash,
            chunkKey=makeChunkKeyForBranchIndex(modelSetKey, newBranch.key),
            packedJson=branchJson,
        )
        inserts.append(existingObject.tupleToSqlaBulkInsertDict())

        chunkKeysForQueue.add((modelSetId, existingObject.chunkKey))

    # 1) Delete existing branches
    if importHashSet:
        # Make note of the IDs being deleted
        # FIXME : Unused
        branchIndexIdsBeingDeleted = [
            item.id
            for item in conn.execute(
                select(
                    distinct=True,
                    columns=[branchIndexTable.c.id],
                    whereclause=branchIndexTable.c.importGroupHash.in_(importHashSet),
                )
            )
        ]

        conn.execute(
            branchIndexTable.delete(
                branchIndexTable.c.importGroupHash.in_(importHashSet)
            )
        )

    # 2) Insert new branches
    if inserts:
        conn.execute(branchIndexTable.insert(), inserts)

    # 3) Queue chunks for recompile
    if chunkKeysForQueue:
        conn.execute(
            queueTable.insert(),
            [dict(modelSetId=m, chunkKey=c) for m, c in chunkKeysForQueue],
        )

    logger.debug(
        "Inserted %s queued %s chunks in %s",
        len(inserts),
        len(chunkKeysForQueue),
        (datetime.now(pytz.utc) - startTime),
    )



"""Added polyline startKey endKey

Peek Plugin Database Migration Script

Revision ID: 558bc7fc4d36
Revises: 24b83c5e7e31
Create Date: 2017-09-23 14:34:32.435996

"""

# revision identifiers, used by Alembic.
revision = "558bc7fc4d36"
down_revision = "24b83c5e7e31"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.add_column(
        "DispPolyline",
        sa.Column("endKey", sa.String(length=50), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "DispPolyline",
        sa.Column("startKey", sa.String(length=50), nullable=True),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolyline_endKey",
        "DispPolyline",
        ["endKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolyline_startKey",
        "DispPolyline",
        ["startKey"],
        unique=False,
        schema="pl_diagram",
    )


def downgrade():
    op.drop_index(
        "idx_DispPolyline_startKey", table_name="DispPolyline", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispPolyline_endKey", table_name="DispPolyline", schema="pl_diagram"
    )
    op.drop_column("DispPolyline", "startKey", schema="pl_diagram")
    op.drop_column("DispPolyline", "endKey", schema="pl_diagram")



import logging
from typing import Dict

from peek_plugin_diagram._private.storage.Display import (
    DispTextStyle,
    DispLineStyle,
    DispColor,
    DispLevel,
    DispLayer,
    DispGroupPointer,
)
from peek_plugin_diagram.tuples.model.ImportLiveDbDispLinkTuple import (
    ImportLiveDbDispLinkTuple,
)
from peek_plugin_livedb.tuples.ImportLiveDbItemTuple import ImportLiveDbItemTuple
from sqlalchemy import select

# NO_SYMBOL = "NO_SYMBOL"
#
# DRESSING_GRID_KEY = "dressings"
# DRESSING_COORD_SET_NAME = "dressings"

logger = logging.getLogger(__name__)

textTable = DispTextStyle.__table__
lineTable = DispLineStyle.__table__
colorTable = DispColor.__table__
levelTable = DispLevel.__table__
layerTable = DispLayer.__table__


class LookupHashConverter:
    def __init__(self, ormSession, modelSetId: int, coordSetId: int):
        self._ormSession = ormSession
        self._modelSetId = modelSetId
        self._coordSetId = coordSetId

        assert modelSetId is not None, "modelSetId is None"
        assert coordSetId is not None, "coordSetId is None"

        self._textStyleIdByImportHash = self._loadLookupByModelSet(textTable)
        self._lineStyleIdByImportHash = self._loadLookupByModelSet(lineTable)
        self._colorIdByImportHash = self._loadLookupByModelSet(colorTable)
        self._levelByImportHash = self._loadLookupByCoordSet(levelTable)
        self._layerByImportHash = self._loadLookupByModelSet(layerTable)

    def _loadLookupByModelSet(self, table) -> Dict[str, int]:

        resultSet = self._ormSession.execute(
            select([table.c.importHash, table.c.id]).where(
                table.c.modelSetId == self._modelSetId
            )
        )

        return dict(resultSet.fetchall())

    def _loadLookupByCoordSet(self, table) -> Dict[str, int]:

        resultSet = self._ormSession.execute(
            select([table.c.importHash, table.c.id]).where(
                table.c.coordSetId == self._coordSetId
            )
        )

        return dict(resultSet.fetchall())

    def convertLookups(self, disp):

        T = ImportLiveDbDispLinkTuple

        colourFields = {
            T.DISP_ATTR_FILL_COLOR,
            T.DISP_ATTR_COLOR,
            T.DISP_ATTR_LINE_COLOR,
            T.DISP_ATTR_EDGE_COLOR,
        }

        for attrName in colourFields:
            if not hasattr(disp, attrName) or getattr(disp, attrName) is None:
                continue
            setattr(disp, attrName, self.getColourId(getattr(disp, attrName)))

        for attrName in (T.DISP_ATTR_LINE_STYLE,):
            if not hasattr(disp, attrName) or getattr(disp, attrName) is None:
                continue
            setattr(disp, attrName, self.getLineStyleId(getattr(disp, attrName)))

        for attrName in (T.DISP_ATTR_TEXT_STYLE,):
            if not hasattr(disp, attrName) or getattr(disp, attrName) is None:
                continue
            setattr(disp, attrName, self.getTextStyleId(getattr(disp, attrName)))

        for attrName in ["levelId"]:
            if not hasattr(disp, attrName) or getattr(disp, attrName) is None:
                continue
            setattr(disp, attrName, self.getLevelId(getattr(disp, attrName)))

        for attrName in ["layerId"]:
            if not hasattr(disp, attrName) or getattr(disp, attrName) is None:
                continue
            setattr(disp, attrName, self.getLayerId(getattr(disp, attrName)))

        # if isinstance(disp, DispGroupPointer):
        #     raise NotImplementedError("Not implemented")

    def getTextStyleId(self, importHash):
        importHash = str(importHash)

        if importHash in self._textStyleIdByImportHash:
            return self._textStyleIdByImportHash[importHash]

        textStyle = DispTextStyle()
        textStyle.modelSetId = self._modelSetId
        textStyle.name = "Peek Created %s" % importHash
        textStyle.importHash = importHash

        self._ormSession.add(textStyle)
        self._ormSession.commit()
        self._textStyleIdByImportHash[importHash] = textStyle.id
        return textStyle.id

    def getLineStyleId(self, importHash):
        importHash = str(importHash)

        if importHash in self._lineStyleIdByImportHash:
            return self._lineStyleIdByImportHash[importHash]

        newLine = DispLineStyle()
        newLine.modelSetId = self._modelSetId
        newLine.name = "Peek Created %s" % importHash
        newLine.capStyle = "butt"
        newLine.joinStyle = "miter"
        newLine.winStyle = 1
        newLine.importHash = importHash

        self._ormSession.add(newLine)
        self._ormSession.commit()
        self._lineStyleIdByImportHash[importHash] = newLine.id
        return newLine.id

    def getColourId(self, importHash):
        if importHash is None:
            return None

        importHash = str(importHash)

        if importHash in self._colorIdByImportHash:
            return self._colorIdByImportHash[importHash]

        newColor = DispColor()
        newColor.modelSetId = self._modelSetId
        newColor.name = "Peek Created %s" % importHash
        newColor.color = "#ffffff"
        newColor.importHash = importHash

        self._ormSession.add(newColor)
        self._ormSession.commit()
        self._colorIdByImportHash[importHash] = newColor.id
        return newColor.id

    def getLevelId(self, importHash, defaultOrder=None):
        importHash = str(importHash)

        if importHash in self._levelByImportHash:
            return self._levelByImportHash[importHash]

        newLevel = DispLevel()
        newLevel.coordSetId = self._coordSetId
        newLevel.name = "Peek Created %s" % importHash
        newLevel.importHash = importHash
        newLevel.minZoom = 0
        newLevel.maxZoom = 10000

        if defaultOrder is not None:
            newLevel.order = defaultOrder
        else:
            try:
                newLevel.order = int(importHash)
            except ValueError:
                newLevel.order = 0

        self._ormSession.add(newLevel)
        self._ormSession.commit()
        self._levelByImportHash[importHash] = newLevel.id
        return newLevel.id

    def getLayerId(self, importHash, defaultOrder=None):
        importHash = str(importHash)

        if importHash in self._layerByImportHash:
            return self._layerByImportHash[importHash]

        newLayer = DispLayer()
        newLayer.modelSetId = self._modelSetId
        newLayer.name = "Peek Created %s" % importHash
        newLayer.importHash = importHash

        if defaultOrder is not None:
            newLayer.order = defaultOrder
        else:
            try:
                newLayer.order = int(importHash)
            except ValueError:
                newLayer.order = 0

        self._ormSession.add(newLayer)
        self._ormSession.commit()
        self._layerByImportHash[importHash] = newLayer.id
        return newLayer.id

    # ---------------------------------------------------------------
    # Live DB Value Translations



from collections import defaultdict

import logging
import typing
from sqlalchemy import select
from typing import List, Dict

from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_diagram._private.storage.ModelSet import ModelSet, ModelCoordSet

logger = logging.getLogger(__name__)

_modelSetTable = ModelSet.__table__
_coordSetTable = ModelCoordSet.__table__


def getModelSetIdCoordSetId(
    modelSetKeyCoordSetKey: List[typing.Tuple[str, str]]
) -> Dict[typing.Tuple[str, str], typing.Tuple[int, int]]:
    """Get Coord Set Ids

    Given a tuple of (ModelSet.key, CoordSet.key), return with that as the key
    and the CoordSet.id as the value

    """

    results: Dict[typing.Tuple[str, str], typing.Tuple[int, int]] = {}

    coordSetKeysByModelSetKey = defaultdict(list)
    for modelSetKey, coordSetKey in modelSetKeyCoordSetKey:
        coordSetKeysByModelSetKey[modelSetKey].append(coordSetKey)

    modelSetIdByKey = _loadModelSets()

    for modelSetKey, coordSetKeys in coordSetKeysByModelSetKey.items():
        modelSetId = modelSetIdByKey.get(modelSetKey)
        if modelSetId is None:
            modelSetId = _makeModelSet(modelSetKey)
            modelSetIdByKey[modelSetKey] = modelSetId

        coordSetIdByKey = _loadCoordSets(modelSetId)

        for coordSetKey in coordSetKeys:
            coordSetId = modelSetIdByKey.get(coordSetKey)
            if coordSetId is None:
                coordSetId = _makeCoordSet(modelSetId, coordSetKey)
                coordSetIdByKey[coordSetKey] = coordSetId

            results[(modelSetKey, coordSetKey)] = (modelSetId, coordSetId)

    return results


def _loadModelSets() -> Dict[str, int]:
    # Get the model set
    engine = CeleryDbConn.getDbEngine()
    conn = engine.connect()
    try:
        results = list(
            conn.execute(select(columns=[_modelSetTable.c.id, _modelSetTable.c.key]))
        )
        modelSetIdByKey = {o.key: o.id for o in results}
        del results

    finally:
        conn.close()
    return modelSetIdByKey


def _makeModelSet(modelSetKey: str) -> int:
    # Get the model set
    dbSession = CeleryDbConn.getDbSession()
    try:
        newItem = ModelSet(key=modelSetKey, name=modelSetKey)
        dbSession.add(newItem)
        dbSession.commit()
        return newItem.id

    finally:
        dbSession.close()


def _loadCoordSets(modelSetId: int) -> Dict[str, int]:
    # Get the model set
    engine = CeleryDbConn.getDbEngine()
    conn = engine.connect()
    try:
        results = list(
            conn.execute(
                select(
                    columns=[_coordSetTable.c.id, _coordSetTable.c.key],
                    whereclause=_coordSetTable.c.modelSetId == modelSetId,
                )
            )
        )
        coordSetIdByKey = {o.key: o.id for o in results}
        del results

    finally:
        conn.close()

    return coordSetIdByKey


def _makeCoordSet(modelSetId: int, coordSetKey: str) -> int:
    # Make a coord set
    dbSession = CeleryDbConn.getDbSession()
    try:
        newItem = ModelCoordSet(
            modelSetId=modelSetId, key=coordSetKey, name=coordSetKey
        )
        dbSession.add(newItem)
        dbSession.commit()
        return newItem.id

    finally:
        dbSession.close()



import logging
from os import path as osp

from twisted.internet.defer import inlineCallbacks

from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_plugin_base.client.PluginClientEntryHookABC import PluginClientEntryHookABC
from peek_plugin_diagram._private.PluginNames import diagramActionProcessorName
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.PluginNames import diagramObservableName
from peek_plugin_diagram._private.client.TupleDataObservable import (
    makeClientTupleDataObservableHandler,
)
from peek_plugin_diagram._private.client.controller.BranchIndexCacheController import (
    BranchIndexCacheController,
)
from peek_plugin_diagram._private.client.controller.CoordSetCacheController import (
    CoordSetCacheController,
)
from peek_plugin_diagram._private.client.controller.GridCacheController import (
    GridCacheController,
)
from peek_plugin_diagram._private.client.controller.LocationIndexCacheController import (
    LocationIndexCacheController,
)
from peek_plugin_diagram._private.client.controller.LookupCacheController import (
    LookupCacheController,
)
from peek_plugin_diagram._private.client.controller.ModelSetCacheController import (
    ModelSetCacheController,
)
from peek_plugin_diagram._private.client.handlers.BranchIndexCacheHandler import (
    BranchIndexCacheHandler,
)
from peek_plugin_diagram._private.client.handlers.GridCacheHandler import (
    GridCacheHandler,
)
from peek_plugin_diagram._private.client.handlers.LocationIndexCacheHandler import (
    LocationIndexCacheHandler,
)
from peek_plugin_diagram._private.storage.DeclarativeBase import loadStorageTuples
from peek_plugin_diagram._private.tuples import loadPrivateTuples
from peek_plugin_diagram.tuples import loadPublicTuples
from txhttputil.site.FileUnderlayResource import FileUnderlayResource
from vortex.handler.TupleActionProcessorProxy import TupleActionProcessorProxy
from vortex.handler.TupleDataObservableProxyHandler import (
    TupleDataObservableProxyHandler,
)
from vortex.handler.TupleDataObserverClient import TupleDataObserverClient

logger = logging.getLogger(__name__)


class ClientEntryHook(PluginClientEntryHookABC):
    def __init__(self, *args, **kwargs):
        """" Constructor """
        # Call the base classes constructor
        PluginClientEntryHookABC.__init__(self, *args, **kwargs)

        #: Loaded Objects, This is a list of all objects created when we start
        self._loadedObjects = []

    def load(self) -> None:
        """Load

        This will be called when the plugin is loaded, just after the db is migrated.
        Place any custom initialiastion steps here.

        """

        loadStorageTuples()

        loadPrivateTuples()
        loadPublicTuples()

        logger.debug("Loaded")

    @inlineCallbacks
    def start(self):
        """Load

        This will be called when the plugin is loaded, just after the db is migrated.
        Place any custom initialiastion steps here.

        """

        # Proxy actions back to the server, we don't process them at all
        self._loadedObjects.append(
            TupleActionProcessorProxy(
                tupleActionProcessorName=diagramActionProcessorName,
                proxyToVortexName=peekServerName,
                additionalFilt=diagramFilt,
            )
        )

        # Provide the devices access to the servers observable
        tupleDataObservableProxyHandler = TupleDataObservableProxyHandler(
            observableName=diagramObservableName,
            proxyToVortexName=peekServerName,
            additionalFilt=diagramFilt,
            observerName="Proxy to devices",
        )
        self._loadedObjects.append(tupleDataObservableProxyHandler)

        #: This is an observer for us (the client) to use to observe data
        # from the server
        serverTupleObserver = TupleDataObserverClient(
            observableName=diagramObservableName,
            destVortexName=peekServerName,
            additionalFilt=diagramFilt,
            observerName="Data for us",
        )
        self._loadedObjects.append(serverTupleObserver)

        # -----

        # Buffer the lookups in the client (us)
        lookupCacheController = LookupCacheController(serverTupleObserver)
        self._loadedObjects.append(lookupCacheController)

        # Buffer the model sets in the client (us)
        modelSetCacheController = ModelSetCacheController(serverTupleObserver)
        self._loadedObjects.append(modelSetCacheController)

        # Buffer the coord sets in the client (us)
        coordSetCacheController = CoordSetCacheController(serverTupleObserver)
        self._loadedObjects.append(coordSetCacheController)

        # ----- Grid Cache Controller
        gridCacheController = GridCacheController(self.platform.serviceId)
        self._loadedObjects.append(gridCacheController)

        # This is the custom handler for the client
        gridCacheHandler = GridCacheHandler(
            cacheController=gridCacheController, clientId=self.platform.serviceId
        )
        self._loadedObjects.append(gridCacheHandler)

        gridCacheController.setCacheHandler(gridCacheHandler)

        # ----- Location Index Cache Controller

        locationIndexCacheController = LocationIndexCacheController(
            self.platform.serviceId
        )
        self._loadedObjects.append(locationIndexCacheController)

        # This is the custom handler for the client
        locationIndexCacheHandler = LocationIndexCacheHandler(
            cacheController=locationIndexCacheController,
            clientId=self.platform.serviceId,
        )
        self._loadedObjects.append(locationIndexCacheHandler)

        locationIndexCacheController.setCacheHandler(locationIndexCacheHandler)

        # ----------------
        # BranchIndex Cache Controller

        branchIndexCacheController = BranchIndexCacheController(self.platform.serviceId)
        self._loadedObjects.append(branchIndexCacheController)

        # ----------------
        # BranchIndex Cache Handler

        branchIndexHandler = BranchIndexCacheHandler(
            cacheController=branchIndexCacheController, clientId=self.platform.serviceId
        )
        self._loadedObjects.append(branchIndexHandler)
        branchIndexCacheController.setCacheHandler(branchIndexHandler)

        # -----
        # Create the Tuple Observer
        tupleObservable = makeClientTupleDataObservableHandler(
            tupleDataObservableProxyHandler,
            modelSetCacheController,
            coordSetCacheController,
            gridCacheController,
            lookupCacheController,
            locationIndexCacheController,
            branchIndexCacheController,
        )
        # This is already in the _loadedObjects, it's tupleDataObservableProxyHandler

        # ----------------
        # Set the caches reference to the handler
        lookupCacheController.setTupleObservable(tupleObservable)
        modelSetCacheController.setTupleObservable(tupleObservable)
        coordSetCacheController.setTupleObservable(tupleObservable)

        yield locationIndexCacheController.start()
        yield branchIndexCacheController.start()
        yield gridCacheController.start()
        lookupCacheController.start()
        modelSetCacheController.start()
        coordSetCacheController.start()

        # THIS WILL BE USED WHEN THE DEVICE IS RUNNING NS LIVE SYNC
        # Add in the HTTP resource that allows images to be downloaded
        resource = FileUnderlayResource()
        resource.enableSinglePageApplication()

        self.platform.addFieldResource(b"web_dist", resource)

        logger.debug("Started")

    def stop(self):
        """Stop

        This method is called by the platform to tell the peek app to shutdown and stop
        everything it's doing
        """
        # Shutdown and dereference all objects we constructed when we started
        while self._loadedObjects:
            self._loadedObjects.pop().shutdown()

        logger.debug("Stopped")

    def unload(self):
        """Unload

        This method is called after stop is called, to unload any last resources
        before the PLUGIN is unlinked from the platform

        """
        logger.debug("Unloaded")



"""Added default group ptr to coord set

Peek Plugin Database Migration Script

Revision ID: aa76b059b81e
Revises: d66cda728b26
Create Date: 2019-05-26 16:00:44.664243

"""

# revision identifiers, used by Alembic.
revision = "aa76b059b81e"
down_revision = "f4dce3e782ec"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.add_column(
        "DispGroupPointer",
        sa.Column(
            "targetDispGroupName", sa.String(), server_default="0", nullable=True
        ),
        schema="pl_diagram",
    )

    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultEdgeCoordSetId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultEdgeGroupName", sa.String(), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultVertexCoordSetId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "ModelCoordSet",
        sa.Column("editDefaultVertexGroupName", sa.String(), nullable=True),
        schema="pl_diagram",
    )
    op.create_index(
        "idxCoordModel_editDefaultEdgeCoordSetId",
        "ModelCoordSet",
        ["editDefaultEdgeCoordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxCoordModel_editDefaultVertexCoordSetId",
        "ModelCoordSet",
        ["editDefaultVertexCoordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_foreign_key(
        "ModelCoordSet_editDefaultEdgeCoordSetId_fkey",
        "ModelCoordSet",
        "ModelCoordSet",
        ["editDefaultEdgeCoordSetId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )
    op.create_foreign_key(
        "ModelCoordSet_editDefaultVertexCoordSetId_fkey",
        "ModelCoordSet",
        "ModelCoordSet",
        ["editDefaultVertexCoordSetId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "ModelCoordSet_editDefaultVertexCoordSetId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_constraint(
        "ModelCoordSet_editDefaultEdgeCoordSetId_fkey",
        "ModelCoordSet",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_index(
        "idxCoordModel_editDefaultVertexCoordSetId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_index(
        "idxCoordModel_editDefaultEdgeCoordSetId",
        table_name="ModelCoordSet",
        schema="pl_diagram",
    )
    op.drop_column("ModelCoordSet", "editDefaultVertexGroupName", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultVertexCoordSetId", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultEdgeGroupName", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "editDefaultEdgeCoordSetId", schema="pl_diagram")
    op.drop_column("DispGroupPointer", "name", schema="pl_diagram")
    # ### end Alembic commands ###



import logging
from typing import List, Dict

from sqlalchemy import select

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkUpdateHandlerABC import (
    ACIChunkUpdateHandlerABC,
)
from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_plugin_diagram._private.client.controller.LocationIndexCacheController import (
    clientLocationIndexUpdateFromServerFilt,
)
from peek_plugin_diagram._private.storage.LocationIndex import LocationIndexCompiled
from peek_plugin_diagram._private.storage.ModelSet import ModelSet

logger = logging.getLogger(__name__)


class ClientLocationIndexUpdateHandler(ACIChunkUpdateHandlerABC):
    _ChunkedTuple: ACIEncodedChunkTupleABC = LocationIndexCompiled
    _updateFromServerFilt: Dict = clientLocationIndexUpdateFromServerFilt
    _logger: logging.Logger = logger

    @classmethod
    def _makeLoadSql(cls, chunkKeys: List[str]):
        chunkTable = LocationIndexCompiled.__table__
        msTable = ModelSet.__table__

        return (
            select(
                [
                    chunkTable.c.indexBucket,
                    chunkTable.c.blobData,
                    chunkTable.c.lastUpdate,
                    msTable.c.key,
                ]
            )
            .select_from(chunkTable.join(msTable))
            .where(chunkTable.c.indexBucket.in_(chunkKeys))
        )



from typing import List

from peek_plugin_livedb.tuples.LiveDbDisplayValueTuple import LiveDbDisplayValueTuple


class WorkerApi:
    """Worker Api

    This class allows other classes to work with the LiveDB plugin on the
    worker service.

    """

    _FETCH_SIZE = 5000

    @classmethod
    def updateLiveDbDisplayValues(
        cls,
        ormSession,
        modelSetKey: str,
        liveDbRawValues: List[LiveDbDisplayValueTuple],
    ) -> None:
        """Convert Live DB Display Values

        Return an array of items representing the display values from the LiveDB.

        :param liveDbRawValues:
        :param ormSession: The SQLAlchemy orm session from the calling code.
        :param modelSetKey: The name of the model set to get the keys for

        :returns: The input list with populated display values.
        """
        from peek_plugin_diagram._private.worker.api.WorkerApiImpl import WorkerApiImpl

        WorkerApiImpl.updateLiveDbDisplayValues(
            ormSession, modelSetKey, liveDbRawValues
        )

    @classmethod
    def liveDbDisplayValueUpdateNotify(
        cls, ormSession, modelSetKey: str, updatedKeys: List[str]
    ) -> None:
        """LiveDB Display Value Update Notify

        This method is called by the LiveDB when the LiveDB display values update.
        This API support is the worker because the LiveDB updates are applied
        in the worker as well.

        THIS METHOD DOES NOT COMMIT THE TRANSACTION.

        :param ormSession: The SQLAlchemy orm session from the calling code.
        :param modelSetKey: The name of the model set to get the keys for
        :param updatedKeys: An array of LiveDb item keys that have been updated

        :returns: None
        """
        from peek_plugin_diagram._private.worker.api.WorkerApiImpl import WorkerApiImpl

        return WorkerApiImpl.liveDbDisplayValueUpdateNotify(
            ormSession, modelSetKey, updatedKeys
        )



"""Added textHeight to DispText

Peek Plugin Database Migration Script

Revision ID: d6e25d8ba156
Revises: 2127d894e46b
Create Date: 2017-08-14 13:27:08.675775

"""

# revision identifiers, used by Alembic.
revision = "d6e25d8ba156"
down_revision = "2127d894e46b"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.add_column(
        "DispText",
        sa.Column("textHeight", sa.Float(), nullable=True),
        schema="pl_diagram",
    )

    op.add_column(
        "DispText",
        sa.Column("textHStretch", sa.Float(), nullable=True, server_default="1.0"),
        schema="pl_diagram",
    )

    op.execute(
        """
    UPDATE "pl_diagram"."DispText"
    SET "textHStretch" = 1.0
    """
    )

    op.alter_column(
        "DispText",
        "textHStretch",
        existing_type=sa.Float(),
        nullable=False,
        schema="pl_diagram",
    )


def downgrade():
    raise NotImplementedError("Downgrade is not implemented")



"""Fixed importHash indexes for lookups

Peek Plugin Database Migration Script

Revision ID: 9e7ca1acd6be
Revises: 7eed363f5df5
Create Date: 2017-08-12 12:30:43.048362

"""

# revision identifiers, used by Alembic.
revision = "9e7ca1acd6be"
down_revision = "7eed363f5df5"
branch_labels = None
depends_on = None

from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_DispColor_importHash", table_name="DispColor", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispColor_modelSetId", table_name="DispColor", schema="pl_diagram"
    )

    op.drop_index(
        "idx_DispLayer_modelSetId", table_name="DispLayer", schema="pl_diagram"
    )

    op.drop_index(
        "idx_DispLevel_coordSetId", table_name="DispLevel", schema="pl_diagram"
    )

    op.drop_index(
        "idx_DispLineStyle_importHash", table_name="DispLineStyle", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispLineStyle_modelSetId", table_name="DispLineStyle", schema="pl_diagram"
    )

    op.drop_index(
        "idx_DispTextStyle_importHash", table_name="DispTextStyle", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispTextStyle_modelSetId", table_name="DispTextStyle", schema="pl_diagram"
    )

    op.create_index(
        "idx_DispColor_importHash",
        "DispColor",
        ["modelSetId", "importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispColor_modelSetId",
        "DispColor",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idx_DispLayer_importHash",
        "DispLayer",
        ["modelSetId", "importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLayer_modelSetId",
        "DispLayer",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idx_DispLevel_coordSetId",
        "DispLevel",
        ["coordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLevel_importHash",
        "DispLevel",
        ["coordSetId", "importHash"],
        unique=True,
        schema="pl_diagram",
    )

    op.create_index(
        "idx_DispLineStyle_importHash",
        "DispLineStyle",
        ["modelSetId", "importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLineStyle_modelSetId",
        "DispLineStyle",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )

    op.create_index(
        "idx_DispTextStyle_importHash",
        "DispTextStyle",
        ["modelSetId", "importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispTextStyle_modelSetId",
        "DispTextStyle",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    raise NotImplementedError("Downgrade is not implemented")



"""added disp zOrder

Peek Plugin Database Migration Script

Revision ID: c88ba8f6761f
Revises: 4925c894757b
Create Date: 2017-11-26 12:52:22.016867

"""

# revision identifiers, used by Alembic.
revision = "c88ba8f6761f"
down_revision = "4925c894757b"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        "DispBase",
        sa.Column("zOrder", sa.Integer(), server_default="0", nullable=True),
        schema="pl_diagram",
    )
    op.execute(""" UPDATE "pl_diagram"."DispBase" SET "zOrder" = 0 """)
    op.alter_column(
        "DispBase", "zOrder", type_=sa.Integer, nullable=False, schema="pl_diagram"
    )


def downgrade():
    op.drop_column("DispBase", "zOrder", schema="pl_diagram")



"""added branch tables

Peek Plugin Database Migration Script

Revision ID: f76791d27f6d
Revises: 5e2f84d757f2
Create Date: 2019-02-04 19:34:07.351930

"""

# revision identifiers, used by Alembic.
revision = "f76791d27f6d"
down_revision = "5e2f84d757f2"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "BranchIndexCompilerQueue",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("modelSetId", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("chunkKey", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "chunkKey"),
        schema="pl_diagram",
    )
    op.create_table(
        "BranchIndexEncodedChunk",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("chunkKey", sa.String(), nullable=False),
        sa.Column("encodedData", sa.LargeBinary(), nullable=False),
        sa.Column("encodedHash", sa.String(), nullable=False),
        sa.Column("lastUpdate", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_BranchIndexEnc_modelSetId_chunkKey",
        "BranchIndexEncodedChunk",
        ["modelSetId", "chunkKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "BranchIndex",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.Column("importGroupHash", sa.String(), nullable=True),
        sa.Column("importHash", sa.String(), nullable=True),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("chunkKey", sa.String(), nullable=False),
        sa.Column("packedJson", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_BranchIndex_chunkKey",
        "BranchIndex",
        ["chunkKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_BranchIndex_importGroupHash",
        "BranchIndex",
        ["coordSetId", "importGroupHash"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_BranchIndex_importHash",
        "BranchIndex",
        ["coordSetId", "importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_BranchIndex_key",
        "BranchIndex",
        ["coordSetId", "key"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_table(
        "BranchGridIndex",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("branchIndexId", sa.Integer(), nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.Column("gridKey", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["branchIndexId"], ["pl_diagram.BranchIndex.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "coordSetId"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_BranchGridIndex_coord_key",
        "BranchGridIndex",
        ["coordSetId", "branchIndexId", "gridKey"],
        unique=True,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_BranchGridIndex_coord_key",
        table_name="BranchGridIndex",
        schema="pl_diagram",
    )
    op.drop_table("BranchGridIndex", schema="pl_diagram")
    op.drop_index("idx_BranchIndex_key", table_name="BranchIndex", schema="pl_diagram")
    op.drop_index(
        "idx_BranchIndex_importGroupHash", table_name="BranchIndex", schema="pl_diagram"
    )
    op.drop_index(
        "idx_BranchIndex_chunkKey", table_name="BranchIndex", schema="pl_diagram"
    )
    op.drop_table("BranchIndex", schema="pl_diagram")
    op.drop_index(
        "idx_BranchIndexEnc_modelSetId_chunkKey",
        table_name="BranchIndexEncodedChunk",
        schema="pl_diagram",
    )
    op.drop_table("BranchIndexEncodedChunk", schema="pl_diagram")
    op.drop_table("BranchIndexCompilerQueue", schema="pl_diagram")
    # ### end Alembic commands ###



"""added line templates

Peek Plugin Database Migration Script

Revision ID: ec97d31aebb0
Revises: a73574690197
Create Date: 2019-09-14 20:00:19.116158

"""

# revision identifiers, used by Alembic.
revision = "ec97d31aebb0"
down_revision = "a73574690197"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "DispEdgeTemplate",
        sa.Column("id", sa.Integer(), autoincrement=False, nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("lineWidth", sa.Integer(), server_default="2", nullable=False),
        sa.Column("lineColorId", sa.Integer(), nullable=True),
        sa.Column("lineStyleId", sa.Integer(), nullable=True),
        sa.Column("startEndType", sa.Integer(), nullable=True),
        sa.Column("endEndType", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["lineColorId"],
            ["pl_diagram.DispColor.id"],
        ),
        sa.ForeignKeyConstraint(
            ["lineStyleId"],
            ["pl_diagram.DispLineStyle.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("DispEdgeTemplate", schema="pl_diagram")
    # ### end Alembic commands ###
